/*
 * Copyright 2013 The Android Open Source Project
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package androidx.media.filterfw.samples.simplecamera;

import androidx.media.filterfw.Filter;
import androidx.media.filterfw.FrameValue;
import androidx.media.filterfw.FrameType;
import androidx.media.filterfw.MffContext;
import androidx.media.filterfw.MffFilterTestCase;
import androidx.media.filterfw.samples.simplecamera.FloatArrayToStrFilter;

import java.util.concurrent.ExecutionException;
import java.util.concurrent.TimeoutException;


public class FloatArrayToStrFilterTest extends MffFilterTestCase {

    @Override
    protected Filter createFilter(MffContext mffContext) {
        return new FloatArrayToStrFilter(mffContext, "floatArrayToStrFilter");
    }

    public void testToStr() throws Exception {
        FrameValue floatArray = createFrame(FrameType.array(float.class), new int[] { 1 }).
                asFrameValue();
        float[] floatArr = { 10f, 15f, 25f };
        floatArray.setValue(floatArr);

        injectInputFrame("array", floatArray);

        process();

        assertEquals("[10.0, 15.0, 25.0]", (String) getOutputFrame("string").asFrameValue().
                getValue());
    }
}